<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Action\PostRegistration\Login;

// Import framework stuff
use Org\Mxchange\CoreFramework\Action\BaseAction;
use Org\Mxchange\CoreFramework\Action\PerformableAction;
use Org\Mxchange\CoreFramework\Generic\FrameworkException;
use Org\Mxchange\CoreFramework\Request\Requestable;
use Org\Mxchange\CoreFramework\Response\Responseable;

/**
 * A post registration action to login after the registration is done
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2020 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class LoginAfterRegistrationAction extends BaseAction implements PerformableAction {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of this action
	 *
	 * @return	$actionInstance		An instance of this action class
	 */
	public static final function createLoginAfterRegistrationAction () {
		// Get a new instance
		$actionInstance = new LoginAfterRegistrationAction();

		// Return the instance
		return $actionInstance;
	}

	/**
	 * Executes the command with given request and response objects
	 *
	 * @param	$requestInstance	An instance of a class with an Requestable interface
	 * @param	$responseInstance	An instance of a class with an Responseable interface
	 * @return	void
	 */
	public function execute (Requestable $requestInstance, Responseable $responseInstance) {
		// Get a login class from factory
		$loginInstance = LoginFactory::createLoginObjectByRequest($requestInstance);

		// Login the user by the request instance
		$loginInstance->doLogin($requestInstance, $responseInstance);

		// Was the login fine? Then redirect here
		if ($loginInstance->ifLoginWasSuccessfull()) {
			// Try to redirect here
			try {
				// Redirect...
				$responseInstance->redirectToConfiguredUrl('app_login');

				// Stop here
				exit();
			} catch (FrameworkException $e) {
				// Something went wrong here!
				$responseInstance->addFatalMessage($e->getMessage());
			}
		} else {
			// Attach error message to the response
			$responseInstance->addFatalMessage('failed_login_after_registration');
		}
	}

}
