<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Factory\Stack\File;

// Import framework stuff
use Org\Mxchange\CoreFramework\Factory\BaseFactory;
use Org\Mxchange\CoreFramework\Factory\Object\ObjectFactory;
use Org\Mxchange\CoreFramework\Registry\GenericRegistry;

// Import SPL stuff
use \InvalidArgumentException;
use \SplFileInfo;

/**
 * A factory class for file-based stack indexes
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2020 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class FileStackIndexFactory extends BaseFactory {
	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	private function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Returns a singleton (registry-based) StackableFile instance
	 *
	 * @param	$fileInfoInstance	An instance of a SplFileInfo class
	 * @return	$indexInstance	An instance of a IndexableStack class
	 * @throws	InvalidArgumentException	If a parameter is invalid
	 */
	public static final function createFileStackIndexInstance (SplFileInfo $fileInfoInstance, string $type) {
		// If there is no handler?
		if (empty($type)) {
			// Throw IAE
			throw new InvalidArgumentException('Parameter "type" is empty');
		} elseif (GenericRegistry::getRegistry()->instanceExists($type . '_index')) {
			// Get handler from registry
			$indexInstance = GenericRegistry::getRegistry()->getInstance($type . '_index');
		} else {
			// Get the handler instance
			$indexInstance = ObjectFactory::createObjectByConfiguredName($type . '_file_stack_index_class', [$fileInfoInstance]);

			// Add it to the registry
			GenericRegistry::getRegistry()->addInstance($type . '_index', $indexInstance);
		}

		// Return the instance
		return $indexInstance;
	}

}
