<?php
// Same namespace as target class
namespace Org\Mxchange\CoreFramework\Bootstrap;

// Inport framework stuff
use Org\Mxchange\CoreFramework\Console\Tools\ConsoleTools;
use Org\Mxchange\CoreFramework\Generic\NullPointerException;
use Org\Mxchange\CoreFramework\Generic\UnsupportedOperationException;

// Import PHPUnit stuff
use PHPUnit\Framework\Error\Notice;
use PHPUnit\Framework\TestCase;

// Import SPL stuff
use \InvalidArgumentException;

/*
 * Copyright (C) 2017 - 2020 Core Developer Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class FrameworkBootstrapTest extends TestCase {

	/**
	 * Own IP address
	 */
	private static $ipAddress = FALSE;

	/**
	 * Setup test case
	 */
	public function setUp() {
		// Trace message
		//* NOISY-DEBUG: */ printf('[%s:%d]: CALLED!' . PHP_EOL, __METHOD__, __LINE__);

		// Call parent method
		parent::setUp();

		// Trace message
		//* NOISY-DEBUG: */ printf('[%s:%d]: EXIT!' . PHP_EOL, __METHOD__, __LINE__);

	}

	/**
	 * Setup test case
	 */
	public static function setUpBeforeClass() {
		// Trace message
		//* NOISY-DEBUG: */ printf('[%s:%d]: CALLED!' . PHP_EOL, __METHOD__, __LINE__);

		// Call parent method
		parent::setUpBeforeClass();

		// Lookup own IP address
		self::$ipAddress = ConsoleTools::acquireSelfIpAddress();

		// Trace message
		//* NOISY-DEBUG: */ printf('[%s:%d]: EXIT!' . PHP_EOL, __METHOD__, __LINE__);
	}

	/**
	 * Tests setting an empty default timezone
	 */
	public function testSettingEmptyDefaultTimezone () {
		// Will throw this exception
		$this->expectException(InvalidArgumentException::class);

		// Test it
		FrameworkBootstrap::setDefaultTimezone('');
	}

	/**
	 * Tests setting invalid timezone
	 */
	public function testSettingInvalidDefaultTimezone () {
		// Expect Notice
		$this->expectException(Notice::class);

		// Try to set it
		FrameworkBootstrap::setDefaultTimezone('!invalid!');
	}

	/**
	 * Tests setting valid timezone
	 */
	public function testSettingValidDefaultTimezone () {
		// Will be true
		$this->assertTrue(FrameworkBootstrap::setDefaultTimezone('Europe/Berlin'));
	}

	/**
	 * Tests if detectServerAddress() is returning what it should for tests.
	 * This will always be 127.0.0.1.
	 */
	public function testConfigDetectServerAddress () {
		// Call it
		$serverAddress = FrameworkBootstrap::detectServerAddress();

		// Should be the same
		$this->assertEquals(self::$ipAddress, $serverAddress);
	}

	/**
	 * Re-tests if detectServerAddress() is returning what it should for tests.
	 * This will always be 127.0.0.1. This method should not invoke
	 * ConsoleTools's method as the configuration entry is already cached.
	 */
	public function testConfigDetectServerAddressCached () {
		// Call it
		$serverAddress = FrameworkBootstrap::detectServerAddress();

		// Should be the same
		$this->assertEquals(self::$ipAddress, $serverAddress);
	}

}
